@@ -1,25 +1,21 @@
-__version__ = "v0.3.2"
+__version__ = "v0.3.3"
 
 import argparse
 import re
 import os
 import os.path
-import pickle
+import pandas
 import sys
-import subprocess
 import textwrap
 import statistics
-import glob
 import copy
 import csv
 import gzip
 import collections
-from collections import defaultdict, Counter, OrderedDict
-import numpy
-import scipy
-import scipy.interpolate
 from alive_progress import alive_it
 
+from .utils import *
+
 ###################################################################
 # 2021.10.04 added more warnings to merge
 # 2021.11.20 revamped merge to be a two step process allowing for edits
@@ -145,11 +141,8 @@ def main(args):
     elif args[0] in subcommands.keys():
         globals()[args[0]](args[1:])
     else:
-        print(
-            "Unknown subcommand ("
-            + args[0]
-            + ") use -h for the list of subcommands!"
-        )
+        print(f"ERROR: Unknown subcommand '{args[0]}'")
+        print("\u2022 Use '-h' for a list of subcommands.")
         sys.exit(-1)
     sys.exit(0)
 
@@ -194,11 +187,42 @@ def seqrun_stats(args):
     aparser.add_argument(
         "--subgrp", required=False, help="sub grouping for stats"
     )
-
     args = aparser.parse_args(args=args)
+
     for samplesheet in args.samplesheet:
-        samplesetnum = collections.Counter()
-        print("########### FASTQS / READS #################")
+        # Convert header to snake case
+        header_to_snake_case(path=samplesheet, overwrite=True)
+
+        # Initialize a counter for the number of sample sets
+        num_sample_sets = collections.Counter()
+
+        # Initialize the grouping dictionary
+        groupings = {}
+
+        # Load the sample sheet header and data into memory for fast retrieval
+        # Also update the number of sample per sample set
+        sampledict = []
+        header = []
+        with open(samplesheet) as samplefile:
+            dictreader = csv.DictReader(samplefile, delimiter="\t")
+            header = dictreader.fieldnames
+            for row in dictreader:
+                sampledict.append(row)
+                num_sample_sets.update({row["sample_set"]: 1})
+
+        # Error if arguments are not correct
+        if args.maingrp not in header:
+            print("ERROR: '--maingrp' invalid.")
+            print(f"\u2022 User entered '{args.maingrp}'.")
+            print(f"\u2022 Available options: {header}.")
+            exit()
+        if args.subgrp and args.subgrp not in header:
+            print("ERROR: '--subgrp' invaluid.")
+            print(f"\u2022 User entered '{args.maingrp}'.")
+            print(f"\u2022 Available options: {header}.")
+            exit()
+
+        print("####################### FASTQS / READS ########################")
         fastqdir = os.path.dirname(samplesheet) + "/fastq"
         fastqs = os.listdir(fastqdir)
         fqss = [f for f in fastqs if "_R1_" in f]
@@ -208,45 +232,32 @@ def seqrun_stats(args):
         bar = alive_it(fqss)
         for fq in bar:
             reads = 0
-            with gzip.open(fastqdir + "/" + fq, mode="rt") as f:
+            with gzip.open(f"{fastqdir}/{fq}", mode="rt") as f:
                 for line in f:
                     if line.startswith("@"):
                         reads += 1
             fastqlen[fq] = reads
             bar.title("Counting the number of FASTQ reads")
 
-        # Load the table into memory for fast retrieval
-        sampledict = []
-        header = []
-        with open(samplesheet) as samplefile:
-            dictreader = csv.DictReader(samplefile, delimiter="\t")
-            for row in dictreader:
-                sampledict.append(row)
-            header = dictreader.fieldnames
-
-        # Error if arguments are not correct
-        if not args.maingrp in header:
-            print("\nERROR: --maingrp invalid.")
-            print(f"\u2022 User entered '{args.maingrp}'.")
-            print(f"\u2022 Available options: {header}.")
-            exit()
-        if args.subgrp and not args.subgrp in header:
-            print("\nERROR: --subgrp invaluid.")
-            print(f"\u2022 User entered '{args.maingrp}'.")
-            print(f"\u2022 Available options: {header}.")
-            exit()
+        # Save all fastq file names
+        fqfiles = fastqlen.keys()
 
+        # For each row in our data, check if there are multiple fastqs and, if
+        # not, set the read count
         for row in sampledict:
+            # Define identifier to compare fastqs
             fqname = "{}-{}-{}_".format(
                 row["sample_name"], row["sample_set"], row["replicate"]
             )
-            fqfiles = fastqlen.keys()
-            row["Read Count"] = 0
-            fqmatchs = [f for f in fqfiles if fqname in f]
+
+            # Use regex to check if the fastq name appears multiple times
+            regex = re.compile("^" + fqname)
+            fqmatchs = list(filter(regex.match, fqfiles))
 
             if len(fqmatchs) > 1:
+                # Throw an error
                 print(
-                    "ERROR: More than one FASTQ for the sampleset rep.",
+                    "ERROR: More than one FASTQ for the sampleset repetition.",
                     fqmatchs,
                     fqname,
                 )
@@ -258,19 +269,26 @@ def seqrun_stats(args):
                 )
                 exit()
             elif len(fqmatchs) == 1:
-                row["Read Count"] = fastqlen[fqmatchs[0]]
+                # Set read count
+                row["read_count"] = fastqlen[fqmatchs[0]]
 
             # Create groupings
-            groupings = {}
+            # TODO - we don't do anything with the grouping variables
             groupings[row[args.maingrp]] = {}
             if args.subgrp:
                 groupings[row[args.maingrp]][row[args.subgrp]] = 1
 
-        # Create updated samplesheet
+        # If there were undetermined reads let the user know
+        if "Undetermined_S0_R1_001.fastq.gz" in fastqlen:
+            cnt = fastqlen["Undetermined_S0_R1_001.fastq.gz"]
+            print(f"There were {cnt} undetermined reads from demultipelxing.")
+
+        # Create updated sample sheet
         print("Creating an updated samplesheet with read counts.")
         sheet_root = os.path.splitext(samplesheet)[0]
-        with open(f"{sheet_root}_readcnt.tsv", mode="wt") as samplesheetout:
-            header.append("Read Count")
+        read_cnt_path = f"{sheet_root}_readcnt.tsv"
+        with open(read_cnt_path, mode="wt") as samplesheetout:
+            header.append("read_count")
             dictwriter = csv.DictWriter(
                 samplesheetout, delimiter="\t", fieldnames=header
             )
@@ -278,41 +296,38 @@ def seqrun_stats(args):
             for row in sampledict:
                 dictwriter.writerow(row)
 
-        print("########### SAMPLES #################")
-        if "Undetermined_S0_R1_001.fastq.gz" in fastqlen:
-            print(
-                "Undetermined reads from demultipelxing",
-                fastqlen["Undetermined_S0_R1_001.fastq.gz"],
-            )
-        print(f"SAMPLE GROUPINGS AND # SAMPLES: {groupings}")
-                
-        # THIS CODE DOES NOT RUN AS THERE IS NO SAMPLESET VARIABLE DEFINED
-        for sampleset in list(samplesetnum.keys()):
-            fqwithreads = len(samplereads.values())
-            total_reads = sum(samplereads.values())
-            mean_reads = int(total_reads / fqwithreads)
-            median_reads = statistics.median(samplereads.values())
-            quantiles = [mean_reads]
-            if fqwithreads > 1:  # quantiles can handle 1 data point
+        print("########################### SAMPLES ###########################")
+        print("PER SAMPLE SET SUMMARY:")
+        # print(f"SAMPLE GROUPINGS AND # SAMPLES: {groupings}")
+
+        # Print some summary stats on reads for each sample set.
+        # Read the read count file in
+        read_cnt_data = pandas.read_csv(read_cnt_path, sep="\t")
+        for key in num_sample_sets:
+            # Print sample set name and number of samples
+            print(f"Sample set {key} ({num_sample_sets[key]:,} samples):")
+
+            # Filter data to sample set and isolate reads
+            filter_data = read_cnt_data[read_cnt_data.sample_set == key]
+            num_samples = filter_data.shape[0]
+            reads = filter_data.read_count
+
+            # Print summary stats
+            print(f"... TOTAL read pairs: {sum(reads):,}")
+            print(f"... MEAN read pairs: {int(statistics.mean(reads)):,}")
+            print(f"... MEDIAN read pairs: {statistics.median(reads):,}")
+
+            # Deciles
+            if num_samples > 1:
                 quantiles = [
-                    round(q)
-                    for q in statistics.quantiles(samplereads.values(), n=10)
+                    f"{round(q):,}" for q in statistics.quantiles(reads)
                 ]
-                quantiles = [f"{q:,}" for q in quantiles]
-            print(f"... TOTAL read pairs {total_reads:,}")
-            print(f"... MEAN read pairs {mean_reads:,}")
-            print(
-                f"... MEDIAN read pairs RS {median_reads:,}",
-                "... DECILES ",
-                quantiles,
-                "median is 50%",
-            )
-            # print (samplereads)
+                print(f"... QUANTILES for read pairs: {quantiles}")
 
 
-#######################################################################3
-#######################################################################3
-#######################################################################3
+#######################################################################
+#######################################################################
+#######################################################################
 shortDescText = "write fastqs from a mergedsheet.tsv after tsv has been created"
 longDescText = """This can be combined with --skipfastqwrite in the main merge_sampleset
 to allow for more complicated merges that might require further intermediate manipulation
@@ -353,31 +368,25 @@ def merge_sampleset_write_fastq(args):
         action="store_true",
         help="keep running and just skip bad fastas",
     )
-
     args = aparser.parse_args(args=args)
+
     print("CREATING and CLEANING NEW FASTQ DIR:", args.newfastqdir)
-    if len(args.newfastqdir) > 0:
-        os.system("mkdir " + args.newfastqdir)  # to lazy to check if it exists
-        os.system(
-            "rm  " + args.newfastqdir + "/*"
-        )  # too lazy to check if they exist
+    if len(args.newfastqdir) == 1:
+        os.system("mkdir " + args.newfastqdir)
+        os.system(f"rm {args.newfastqdir}/*")  # too lazy to check if exist
     else:
-        print("ERROR: bad directory name (" + args.newfastqdir + ")")
+        print(f"ERROR: bad directory name ({args.newfastqdir})")
         exit()
-    total_bad = 0
-    total = good = 0
+    # total_bad = 0
+    # total = good = 0
 
     with open(args.mergedsheet, newline="") as items_file:
         items_reader = csv.DictReader(items_file, delimiter="\t")
-        items_total = 0
+        # items_total = 0
         for item in items_reader:
             fastqs = item["fastq"].split(",")
-            name = (
-                item["sample_name"]
-                + "-"
-                + item["sample_set"]
-                + "-"
-                + item["replicate"]
+            name = "{}-{}-{}".format(
+                item["sample_name"], item["sample_set"], item["replicate"]
             )
             if not args.skipfastqwrite:
                 print("...", name, "...")
@@ -408,14 +417,14 @@ def merge_sampleset_write_fastq(args):
                 if args.skipbadfastqs:
                     print("WARN bad fasta paths for ", name, "(", fastqs, ")")
                 else:
-                    print("!ERROR!", name, fastqs)
-                    print("Skip bad fastqs with --skipbadfastqs")
+                    print("ERROR: Bad FASTQ", name, fastqs)
+                    print("\u2022 Skip bad FASTQs with '--skipbadfastqs'.")
                     exit(1)
 
 
-#######################################################################3
-#######################################################################3
-#######################################################################3
+#######################################################################
+#######################################################################
+#######################################################################
 shortDescText = (
     "merge a sampleset from multiple samplesheets and sequencing runs"
 )
@@ -488,7 +497,6 @@ def merge_sampleset(args):
         action="append",
         help="add unexpected column name",
     )
-
     aparser.add_argument(
         "--skipfastqwrite",
         action="store_true",
@@ -511,37 +519,37 @@ def merge_sampleset(args):
         help="collapse multiple replicates within a samplesheet (error)",
     )
 
-    ##TODO: set default mergedsheet and fastq directory to set_probe
+    # TODO: set default mergedsheet and fastq directory to set_probe
 
     args = aparser.parse_args(args=args)
     mergeonfields = args.mergeon.split("-")
-    print(args)
-    if args.addcolumn == None:  # this should be handled by argparse but not!
+    if args.addcolumn is None:  # this should be handled by argparse but not!
         args.addcolumn = []
 
+    # Convert header to snake case
+    for f in args.sheet:
+        header_to_snake_case(path=f, overwrite=True)
+
     total_samples = []
     merged = {}
     # replicate={}
-    header = global_samples_tsv_header[:]  # copy this as it will be modified
+    header = global_samples_tsv_header[:]  # copy, modify
     header_optional = global_samples_tsv_header_optional[
         :
-    ]  # copy but shouldn't modify
-    headers_to_rename = global_samples_tsv_oldnames
-    # this will not modify
+    ]  # copy, don't modify
+    headers_to_rename = global_samples_tsv_oldnames  # don't modify
 
     print("CREATING and CLEANING NEW FASTQ DIR:", args.newfastqdir)
-    if len(args.newfastqdir) > 0:
+    if len(args.newfastqdir) == 1:
         os.system("mkdir " + args.newfastqdir)
-        os.system(
-            "rm  " + args.newfastqdir + "/*"
-        )  # too lazy to check if they exist
+        os.system(f"rm {args.newfastqdir}/*")  # too lazy to check if exist
     else:
-        print("ERROR: bad directory name (" + args.newfastqdir + ")")
+        print(f"ERROR: bad directory name ({args.newfastqdir})")
         exit()
 
     print("PROCESSING SAMPLE SHEETS...")
     for thefile in args.sheet:
-        print("LOADING FASTQ DIR", thefile + "...")
+        print(f"LOADING FASTQ DIR {thefile} ...")
         print(header)
 
         fastqdir = os.path.dirname(thefile) + "/fastq"
@@ -563,17 +571,14 @@ def merge_sampleset(args):
                         item.pop(oldname)
                         if items_total == 0:
                             print(
-                                "... ... RENAMED old name ",
-                                oldname,
-                                "changed to ",
-                                headers_to_rename[oldname],
-                            )
+                                "... ... RENAMED old name '{}' to '{}'"
+                            ).format(oldname, headers_to_rename[oldname])
                 if items_total == 0:
                     # process the headerline to see what headers are present
                     # determine if we need to add another line to current header.
                     # print (item)
                     for name in item:
-                        if not name in header:
+                        if name not in header:
                             if (
                                 name in header_optional
                                 or name in args.addcolumn
@@ -581,13 +586,11 @@ def merge_sampleset(args):
                                 print("... ... NOTE: ", name, " added to data ")
                                 header.append(name)
                             else:
+                                print("ERROR: Improper column name.")
                                 print(
-                                    "!ERROR! ("
-                                    + name
-                                    + ") is not an proper column name.  use --addcolumn "
-                                    + name
-                                    + " to override."
+                                    f"\u2022 '{name}' is not a proper column name."
                                 )
+                                print("\u2022 Use '--addcolumn' to override.")
                                 exit(1)
 
                 items_total += 1
@@ -597,13 +600,9 @@ def merge_sampleset(args):
                     continue
                 # keep item with proper probeset and sampleset
                 items_kept += 1
-                startfastqname = (
-                    item["sample_name"]
-                    + "-"
-                    + item["sample_set"]
-                    + "-"
-                    + item["replicate"]
-                    + "_"
+
+                startfastqname = "{}-{}-{}_".format(
+                    item["sample_name"], item["sample_set"], item["replicate"]
                 )
                 item["samplelist"] = thefile
                 item["replicate_old"] = item["replicate"]
@@ -667,9 +666,10 @@ def merge_sampleset(args):
         # print (count, mkey, "records:",len(mergeset))
         mergeditem = None
         name = None
-        ### create a merge record from first one and add the others ###
+
+        # Create a merge record from first one and add the others
         for item in mergeset:
-            if mergeditem == None:
+            if mergeditem is None:
                 mergeditem = copy.deepcopy(item)
                 # make sure merged item has all the potential names from header
                 # set file name and replicated based on new name and sampleset#
@@ -685,7 +685,8 @@ def merge_sampleset(args):
                         mergeditem[f].append(item[f])
                     else:
                         mergeditem[f].append("NA")
-        ###collapse in mergeitem sampleset, name and mergeon and error if not the same
+
+        # Collapse in mergeitem sampleset, name and mergeon and error if not the same
         for f in ["sample_name", "sample_set", "mergeon"]:
             mergeditem[f] = list(set(mergeditem[f]))
             if len(mergeditem[f]) == 1:
@@ -695,7 +696,8 @@ def merge_sampleset(args):
                     "!ERROR! ", mergeditem[f], "for", f, " is not consistent!"
                 )
                 exit(1)
-        ### set the replicate #####
+
+        # Set the replicate
         name = mergeditem["sample_name"] + "-" + mergeditem["sample_set"]
         mergeditem["replicate_old"] = mergeditem["replicate"]
         mergeditem["replicate"] = mergeditem["replicate"][0]
@@ -714,7 +716,8 @@ def merge_sampleset(args):
                 exit(1)
             else:
                 replicate[name] = 1
-        ### collapse probesets ####
+
+        # Collapse probesets
         mergeditem["probe_set"] = ",".join(mergeditem["probe_set"]).split(",")
         mergeditem["probe_set"] = list(set(mergeditem["probe_set"]))
         mergeditem["probe_set"].sort()
@@ -723,17 +726,14 @@ def merge_sampleset(args):
     print("################")
     print("MERGE THE FASTQS...")
     for mergeditem in merged_samples:
-        ### merge the fastqs ##########
-        ### this is slowwwwwwwww ######
-        # print (mergeditem)
-        fastqname = (
-            mergeditem["sample_name"]
-            + "-"
-            + mergeditem["sample_set"]
-            + "-"
-            + mergeditem["replicate"]
+        # Merge the fastqs
+        # Note that this is slow
+        fastqname = "{}-{}-{}".format(
+            mergeditem["sample_name"],
+            mergeditem["sample_set"],
+            mergeditem["replicate"],
         )
-        writeoperator = " > "  ##initial write
+        writeoperator = " > "  # initial write
         for pair in mergeditem["fastq"]:
             if len(pair) == 0:
                 # print ("emptty value")
@@ -805,7 +805,7 @@ def merge_sampleset(args):
                 print(" could a R1 or R2 fastq file been deleted???")
                 exit(1)
             writeoperator = " >> "  # now appending
-        ### make fields non-redundant (always make probeset)
+        # Make fields non-redundant (always make probeset)
         mergeditem["fastq"] = [
             fq for sublist in mergeditem["fastq"] for fq in sublist
         ]
